@@ -14,7 +14,6 @@
 #include "../Common/NoClones.h"
 
 #include "LanguageModelling/LanguageModel.h"
-#include "DashEdit.h"
 #include "DasherNode.h"
 #include "DasherComponent.h"
 #include "Alphabet/Alphabet.h"
@@ -31,7 +30,7 @@
 #include "FrameRate.h"
 #include <vector>
 #include <deque>
-#include "DasherGameMode.h"
+
 
 
 namespace Dasher {
@@ -74,7 +73,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
     idJapanese = 4
   } LanguageModelID;
 
-  CDasherModel(CEventHandler * pEventHandler, CSettingsStore * pSettingsStore, CDasherInterfaceBase * pDashIface, CAlphIO *pAlphIO);
+  CDasherModel(CEventHandler * pEventHandler, CSettingsStore * pSettingsStore, CDasherInterfaceBase * pDashIface, CAlphIO *pAlphIO, bool bGameMode = false, const std::string &strGameModeText = "");
   ~CDasherModel();
 
   bool WriteLMToFile(const std::string &strFilename) {
@@ -147,10 +146,11 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void Trace() const;           // diagnostics
   //void Learn_symbol(symbol Symbol) {m_languagemodel->learn_symbol(Symbol);} // feed character to language model
 
-  void Tap_on_display(myint, myint, unsigned long Time, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  // evolves the current viewpoint
+  bool Tap_on_display(myint, myint, unsigned long Time, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  // evolves the current viewpoint
 
 /*   void GoTo(double, myint);     // jumps to a new viewpoint */
-   void NewGoTo(myint n1, myint n2, int style);
+   void NewGoTo(myint n1, myint n2);
+   void OldPush(myint iMousex, myint iMousey);
    double Plan_new_goto_coords(int iRxnew, myint mousey, int *iSteps, myint *o1, myint *o2 , myint *n1, myint *n2); 
 
   void Start();                 // initializes the data structure
@@ -194,7 +194,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
     return total_nats;
   }
 
-  myint PlotGoTo(myint MouseX, myint MouseY);
+//  myint PlotGoTo(myint MouseX, myint MouseY);
 
   void EnterText(CLanguageModel::Context Context, std::string TheText) const;
   void LearnText(CLanguageModel::Context Context, std::string * TheText, bool IsMore);
@@ -229,10 +229,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   CDasherNode *Get_node_under_crosshair();    // Needed for Game Mode
   
-  myint GetGameModePointerLoc() {
-    return m_pGameMode->GetDasherCoordOfTarget();
-  }
-  
   CDasherNode *GetRoot( int iType, CDasherNode *pParent, int iLower, int iUpper, void *pUserData ) {
     switch(iType) {
     case 0:
@@ -276,7 +272,22 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   bool CheckForNewRoot(CDasherView *pView);
 
   void ScheduleZoom(int dasherx, int dashery);
+
+  void HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode);
+
+  int ScheduledSteps() {
+    return m_deGotoQueue.size();
+  }
+
+  // Apply an offset to the 'target' coordinates - implements the jumps in
+  // two button dynamic mode.
+  void Offset(int iOffset);
  
+  // Reset the 'target' root coordinates to match those currently visible. 
+  // Appropriate for abrubt changes in behaviour (such as backing off in 
+  // button modes)
+  void MatchTarget(bool bReverse);
+
  private:
 
   CDasherInterfaceBase * m_pDasherInterface;
@@ -293,8 +304,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   /////////////////////////////////////////////////////////////////////////////
 
-  CDasherGameMode *m_pGameMode;
-
   CDasherNode *m_Root;
 
   // Old root notes
@@ -302,6 +311,9 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   // Rootmin and Rootmax specify the position of the root node in Dasher coords
   myint m_Rootmin, m_Rootmax;
+  
+  myint m_iTargetMin;
+  myint m_iTargetMax;
 
   myint m_Rootmin_min, m_Rootmax_max;
 
@@ -321,16 +333,13 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   CDasherNode *Get_node_under_mouse(myint smousex, myint smousey);
 
-  double Get_new_root_coords(myint mousex, myint mousey);
+  void Get_new_root_coords(myint mousex, myint mousey, myint &iNewMin, myint &iNewMax);
 
-  void DoZoom(myint iTargetMin, myint iTargetMax);
-
-  void Get_new_goto_coords(double zoomfactor, myint mousey);
+ // void Get_new_goto_coords(double zoomfactor, myint mousey);
 
   void Get_string_under_mouse(const myint smousex, const myint smousey, std::vector < symbol > &str);
 
-  
-
+  double CorrectionFactor(int dasherx, int dashery);
 
   void Recursive_Push_Node(CDasherNode * pNode, int depth);
 
@@ -339,6 +348,8 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   CAlphabetManagerFactory *m_pAlphabetManagerFactory;
   CControlManagerFactory *m_pControlManagerFactory;
 
+  bool m_bGameMode;
+
 #ifdef JAPANESE
   CConversionManagerFactory *m_pConversionManagerFactory;
 #endif
@@ -350,8 +361,12 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   };
 
   std::deque<SGotoItem> m_deGotoQueue;
+  
+  // Whether characters entered by alphabet manager are expected to
+  // require conversion.
+  // TODO: Need to rethink this at some point.
+  bool m_bRequireConversion;
 
-  friend class CDasherGameMode;
   friend class CTrainer;
   friend class CDasherNode;
 
