@@ -1,6 +1,11 @@
+#include "../../config.h"
+
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#ifdef WITH_MAEMOFULLSCREEN
+#include <hildon-widgets/hildon-program.h>
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
@@ -27,10 +32,13 @@ struct _DasherMainPrivate {
   GtkWidget *pMenuBar;
   GtkWidget *pSideMenu;
   GtkWidget *pDragHandle; 
-  GtkWidget *pOuterFrame;
   GtkWidget *pInnerFrame;
   GtkWidget *pSpeedBox;
   GtkWidget *pAlphabetCombo;
+  GtkWidget *pStatusControl;
+  GtkWidget *pHideArrow;
+
+  GtkWidget *pContextMenu;
 
   GtkListStore *pAlphabetList;
 
@@ -40,6 +48,10 @@ struct _DasherMainPrivate {
 
 #ifdef WITH_MAEMO
   DasherMaemoHelper *pMaemoHelper;
+#ifdef WITH_MAEMOFULLSCREEN
+  HildonProgram *pProgram;
+  HildonWindow *pHWindow;
+#endif
 #endif
 
   // Properties of the window
@@ -63,6 +75,13 @@ struct _DasherMainPrivate {
 
 typedef struct _DasherMainPrivate DasherMainPrivate;
 
+enum {
+  REALIZED,
+  SIGNAL_NUM
+};
+
+static guint dasher_main_signals[SIGNAL_NUM] = { 0 };
+
 // Private member functions
 
 static void dasher_main_class_init(DasherMainClass *pClass);
@@ -81,6 +100,7 @@ static void dasher_main_ungrab(DasherMain *pSelf, GdkEventButton *pEvent);
 static gboolean dasher_main_motion(DasherMain *pSelf, GdkEventMotion *pEvent);
 static gboolean dasher_main_speed_changed(DasherMain *pSelf);
 static void dasher_main_populate_alphabet_combo(DasherMain *pSelf);
+void dasher_main_build_context_menu(DasherMain *pSelf);
 
 // Private functions not in class
 extern "C" gboolean take_real_focus(GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
@@ -115,7 +135,9 @@ GType dasher_main_get_type() {
 
 static void dasher_main_class_init(DasherMainClass *pClass) {
   GObjectClass *pObjectClass = (GObjectClass *) pClass;
-  pObjectClass->finalize = dasher_main_destroy;
+  pObjectClass->finalize = dasher_main_destroy; 
+
+  dasher_main_signals[REALIZED] = g_signal_new("realized", G_TYPE_FROM_CLASS(pClass), static_cast < GSignalFlags > (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION), G_STRUCT_OFFSET(DasherMainClass, realized), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 }
 
 static void dasher_main_init(DasherMain *pDasherControl) {
@@ -149,14 +171,17 @@ void dasher_main_load_interface(DasherMain *pSelf) {
 #ifdef WITH_GPE
   szGladeFilename = PROGDATA "/dashergpe.glade";
 #elif WITH_MAEMO
-  szGladeFilename = "/var/lib/install" PROGDATA "/dashermaemo.glade";
-  //szGladeFilename = PROGDATA "/dashermaemo.glade";
+#ifdef WITH_MAEMOFULLSCREEN
+  //  szGladeFilename = "/var/lib/install" PROGDATA "/dashermaemofullscreen.glade";
+  szGladeFilename = PROGDATA "/dashermaemofullscreen.glade";
+#else
+  //szGladeFilename = "/var/lib/install" PROGDATA "/dashermaemo.glade";
+  szGladeFilename = PROGDATA "/dashermaemo.glade";
+#endif
 #else
   szGladeFilename = PROGDATA "/dasher.glade";
 #endif
 
-  g_message("Glade file is: %s", szGladeFilename);
-
   pPrivate->pGladeXML = glade_xml_new(szGladeFilename, NULL, NULL);
 
   if (!pPrivate->pGladeXML) {
@@ -175,12 +200,23 @@ void dasher_main_load_interface(DasherMain *pSelf) {
   pPrivate->pMenuBar = glade_xml_get_widget(pPrivate->pGladeXML, "dasher_menu_bar");
   pPrivate->pSideMenu = glade_xml_get_widget(pPrivate->pGladeXML, "SideMenu");
   pPrivate->pDragHandle = glade_xml_get_widget(pPrivate->pGladeXML, "button31");
-  pPrivate->pOuterFrame = glade_xml_get_widget(pPrivate->pGladeXML, "OuterFrame");
+  pPrivate->pHideArrow = glade_xml_get_widget(pPrivate->pGladeXML, "hide_arrow");
+
+  // TODO: This could be made more sensible with consistent naming
+#ifdef WITH_MAEMO
+#ifdef WITH_MAEMOFULLSCREEN
+  pPrivate->pInnerFrame = glade_xml_get_widget(pPrivate->pGladeXML, "hpaned1"); 
+#else
+  pPrivate->pInnerFrame = glade_xml_get_widget(pPrivate->pGladeXML, "hbox1"); 
+#endif
+#else
   pPrivate->pInnerFrame = glade_xml_get_widget(pPrivate->pGladeXML, "vbox1"); 
+#endif
 
 #ifndef WITH_MAEMO
   pPrivate->pSpeedBox = glade_xml_get_widget(pPrivate->pGladeXML, "spinbutton1");
   pPrivate->pAlphabetCombo = glade_xml_get_widget(pPrivate->pGladeXML, "combobox1");
+  pPrivate->pStatusControl = glade_xml_get_widget(pPrivate->pGladeXML, "hbox8"); 
 
   pPrivate->pAlphabetList = gtk_list_store_new(1, G_TYPE_STRING);
   gtk_combo_box_set_model(GTK_COMBO_BOX(pPrivate->pAlphabetCombo), 
@@ -195,6 +231,80 @@ void dasher_main_load_interface(DasherMain *pSelf) {
   dasher_main_setup_window_type(pSelf);
 
   gtk_widget_add_events(pPrivate->pDragHandle, GDK_POINTER_MOTION_MASK);
+#else
+
+#ifdef WITH_MAEMOFULLSCREEN
+  // TODO: This is horrible - no need to get it from the glade file if we're not going to use it
+
+
+  pPrivate->pProgram = HILDON_PROGRAM(hildon_program_get_instance());
+  //  hildon_app_set_title(pPrivate->pApp, "Dasher"); 
+
+  pPrivate->pHWindow = HILDON_WINDOW(hildon_window_new());
+  hildon_program_add_window(pPrivate->pProgram, pPrivate->pHWindow);
+
+  gtk_widget_reparent(pPrivate->pInnerFrame, GTK_WIDGET(pPrivate->pHWindow));
+  //  gtk_paned_set_position(GTK_PANED(window), 100);
+
+  /* Do menu setup */
+  GtkMenu *main_menu;
+  GtkWidget *file_menu;
+  GtkWidget *file_menu_item;
+  GtkWidget *options_menu;
+  GtkWidget *options_menu_item;
+  GtkWidget *help_menu;
+  GtkWidget *help_menu_item;
+
+
+//   main_menu = hildon_appview_get_menu(appview);
+
+  main_menu = GTK_MENU(gtk_menu_new());
+  file_menu = glade_xml_get_widget(pPrivate->pGladeXML, "menuitem4_menu");
+  options_menu = glade_xml_get_widget(pPrivate->pGladeXML, "options1_menu");
+  help_menu = glade_xml_get_widget(pPrivate->pGladeXML, "menuitem7_menu");
+  file_menu_item = gtk_menu_item_new_with_label ("File");
+  options_menu_item = gtk_menu_item_new_with_label ("Options");
+  help_menu_item = gtk_menu_item_new_with_label ("Help");
+
+  g_object_ref(file_menu);
+  g_object_ref(options_menu);
+  g_object_ref(help_menu);
+
+  gtk_menu_item_remove_submenu(GTK_MENU_ITEM(glade_xml_get_widget(pPrivate->pGladeXML, "menuitem4")));
+  gtk_menu_item_remove_submenu(GTK_MENU_ITEM(glade_xml_get_widget(pPrivate->pGladeXML, "options1")));
+  gtk_menu_item_remove_submenu(GTK_MENU_ITEM(glade_xml_get_widget(pPrivate->pGladeXML, "menuitem7")));
+
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_menu_item),file_menu);
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(options_menu_item),options_menu); 
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(help_menu_item),help_menu);
+  gtk_menu_append(main_menu, file_menu_item);
+  gtk_menu_append(main_menu, options_menu_item);
+  gtk_menu_append(main_menu, help_menu_item);
+
+  g_object_unref(file_menu);
+  g_object_unref(options_menu);
+  g_object_unref(help_menu);
+
+  hildon_program_set_common_menu(pPrivate->pProgram, main_menu); 
+
+  gtk_widget_show_all( GTK_WIDGET( main_menu ) );
+
+//   /* And toolbar */
+//   GtkWidget *toolbar;
+//   toolbar = glade_xml_get_widget(pPrivate->pGladeXML, "toolbar");
+//   g_print("Got %p\n",toolbar);
+//   gtk_widget_reparent (toolbar, appview->vbox);
+
+  gtk_widget_show_all(GTK_WIDGET(pPrivate->pHWindow));
+
+  gtk_widget_destroy(pPrivate->pMainWindow);
+  pPrivate->pMainWindow = GTK_WIDGET(pPrivate->pHWindow);
+
+  g_signal_connect(G_OBJECT(pPrivate->pHWindow), "delete_event", G_CALLBACK(ask_save_before_exit), NULL);
+
+
+#endif
+
 #endif
 
   pPrivate->bHidden = false;
@@ -208,10 +318,10 @@ void dasher_main_load_interface(DasherMain *pSelf) {
   g_signal_connect(G_OBJECT(pPrivate->pBufferView), "key-press-event", G_CALLBACK(edit_key_press), NULL);
   g_signal_connect(G_OBJECT(pPrivate->pBufferView), "key-release-event", G_CALLBACK(edit_key_release), NULL);
   
-  
+  dasher_main_build_context_menu(pSelf);
 
   // Create a Maemo helper if necessary
-#ifdef WITH_MAEMO
+#if defined WITH_MAEMO && !defined WITH_MAEMOFULLSCREEN
   pPrivate->pMaemoHelper = dasher_maemo_helper_new(GTK_WINDOW(pPrivate->pMainWindow));
 #endif
 
@@ -220,13 +330,14 @@ void dasher_main_load_interface(DasherMain *pSelf) {
   gtk_window_set_decorated(GTK_WINDOW(pPrivate->pMainWindow), false);
 #endif
 }
+
 void dasher_main_setup_window(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
-
+  
   if(dasher_app_settings_get_bool(pPrivate->pAppSettings, BP_GLOBAL_KEYBOARD))
     gdk_window_add_filter(0, keyboard_filter_cb, 0);
-   else
-     gdk_window_remove_filter(0, keyboard_filter_cb, 0);
+  else
+    gdk_window_remove_filter(0, keyboard_filter_cb, 0);
 }
 
 
@@ -248,6 +359,12 @@ void dasher_main_handle_parameter_change(DasherMain *pSelf, int iParameter) {
     else
       gtk_widget_hide(pPrivate->pToolbar);
     break;
+  case BP_SHOW_SLIDER: // TODO: Shouldn't be a core parmeter
+    if( dasher_app_settings_get_bool(pPrivate->pAppSettings, BP_SHOW_SLIDER))
+      gtk_widget_show(pPrivate->pStatusControl);
+    else
+      gtk_widget_hide(pPrivate->pStatusControl);
+    break;
   case APP_SP_EDIT_FONT:
     dasher_main_refresh_font(pSelf);
     break;
@@ -265,6 +382,8 @@ void dasher_main_handle_parameter_change(DasherMain *pSelf, int iParameter) {
     // been mapped, so when the app style is changed the main window
     // needs to be destroyed and recreated.
     {
+      g_message("Setting up window");
+
       GtkWidget *pOldWindow = pPrivate->pMainWindow;
 
       pPrivate->pMainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -305,8 +424,10 @@ void dasher_main_handle_parameter_change(DasherMain *pSelf, int iParameter) {
   case BP_GLOBAL_KEYBOARD:
     dasher_main_setup_window(pSelf);
     break;
-#ifdef WITH_MAEMO
+#if defined WITH_MAEMO && !defined WITH_MAEMOFULLSCREEN
   case APP_LP_MAEMO_SIZE: {
+    g_message("Maemo size");
+
     bool bVisible = GTK_WIDGET_VISIBLE(pPrivate->pMainWindow);
     gtk_widget_hide(pPrivate->pMainWindow);
     if(dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_MAEMO_SIZE) == 0) {
@@ -352,20 +473,16 @@ void dasher_main_set_app_settings(DasherMain *pSelf, DasherAppSettings *pAppSett
   // values
   
 #ifndef WITH_MAEMO
-
   // TODO: put status bar initialisation somewhere else
   pPrivate->iComboCount = 0;
   dasher_main_populate_alphabet_combo(pSelf);
 
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(pPrivate->pSpeedBox), dasher_app_settings_get_long(pPrivate->pAppSettings, LP_MAX_BITRATE) / 100.0);
+#endif
+
+#if !defined WITH_MAEMO || defined WITH_MAEMOFULLSCREEN
   // TODO: bring into object framework
   PopulateMenus(pPrivate->pGladeXML);
-
-
-  if(dasher_app_settings_get_bool(pPrivate->pAppSettings, APP_BP_SHOW_TOOLBAR)) {
-    gtk_widget_show(pPrivate->pToolbar);
-  }
-
   dasher_main_load_state(pSelf);
 #endif
 
@@ -390,7 +507,10 @@ void dasher_main_load_state(DasherMain *pSelf) {
     iWindowHeight = dasher_app_settings_get_long(pPrivate->pAppSettings, APP_LP_SCREEN_HEIGHT_H);
   }
 
+#ifndef WITH_MAEMO
   gtk_window_resize(GTK_WINDOW(pPrivate->pMainWindow), iWindowWidth, iWindowHeight);
+#endif
+
   gtk_paned_set_position(GTK_PANED(pPrivate->pDivider), iEditHeight);
 
   pPrivate->iWidth = iWindowWidth;
@@ -440,15 +560,21 @@ void dasher_main_show(DasherMain *pSelf) {
 }
 
 GtkWidget *dasher_main_create_dasher_control(DasherMain *pSelf) {
-
-  g_message("Creating control");
-
   GtkWidget *pDasherControl = gtk_dasher_control_new();
 
 #ifdef WITH_MAEMO
   gtk_widget_set_size_request(pDasherControl, 175, -1);  
 #endif
 
+
+  // TODO: all of this stuff is pretty pointless
+
+  //  g_message("%d", pSelf);
+
+  //  g_signal_emit_by_name(G_OBJECT(pSelf), "realized");
+
+
+
   return pDasherControl;
 }
 
@@ -508,7 +634,6 @@ void dasher_main_setup_window_position(DasherMain *pSelf) {
 		     &iBytesAfterReturn,
 		     &iData);
 
-  // TODO: need to use width here
   // TODO: need more error checking with raw X11 stuff
   
   iScreenTop = ((unsigned long *)iData)[1];
@@ -517,26 +642,25 @@ void dasher_main_setup_window_position(DasherMain *pSelf) {
   XFree(iData);
 
   
-
-    if(pPrivate->bHidden)
-      iLeft = iScreenWidth - 32; //pPrivate->pSideMenu->allocation.x;
-    else
-      iLeft = iScreenWidth - iTargetWidth - iBuffer;
-
-    if(pPrivate->iPosition < iScreenTop)
-      pPrivate->iPosition = iScreenTop;
-
-    if(pPrivate->iPosition > iScreenTop + iScreenHeight - iTargetHeight)
-      pPrivate->iPosition = iScreenTop + iScreenHeight - iTargetHeight;
- 
-    iTop = pPrivate->iPosition;
-
-    struts[1] = 32;
-    
-    struts[6] = iTop;
-    struts[7] = iTop + iTargetWidth;
-
-    gtk_window_set_gravity(GTK_WINDOW(pPrivate->pMainWindow), GDK_GRAVITY_NORTH_WEST);
+  if(pPrivate->bHidden)
+    iLeft = iScreenWidth - pPrivate->pSideMenu->allocation.width;
+  else
+    iLeft = iScreenWidth - iTargetWidth - iBuffer;
+  
+  if(pPrivate->iPosition < iScreenTop)
+    pPrivate->iPosition = iScreenTop;
+  
+  if(pPrivate->iPosition > iScreenTop + iScreenHeight - iTargetHeight)
+    pPrivate->iPosition = iScreenTop + iScreenHeight - iTargetHeight;
+  
+  iTop = pPrivate->iPosition;
+  
+  struts[1] = pPrivate->pSideMenu->allocation.width;
+  
+  struts[6] = iTop;
+  struts[7] = iTop + iTargetWidth;
+  
+  gtk_window_set_gravity(GTK_WINDOW(pPrivate->pMainWindow), GDK_GRAVITY_NORTH_WEST);
 
   
 //   Atom atom_type[1];
@@ -630,23 +754,20 @@ void dasher_main_setup_window_style(DasherMain *pSelf, bool bTopMost) {
     // TODO: Need to figure out how to hide this sensibly - may involve reparenting and the like
 
 
-//     gtk_widget_reparent(pPrivate->pInnerFrame, pPrivate->pOuterFrame);
-    
-//     gtk_container_add(GTK_CONTAINER(pPrivate->pMainWindow), pPrivate->pOuterFrame);
-//     gtk_widget_reparent(pPrivate->pOuterFrame, pPrivate->pMainWindow);
   }
   else {
     gtk_widget_hide(pPrivate->pSideMenu);
-    gtk_widget_show(pPrivate->pToolbar);
+    if(dasher_app_settings_get_bool(pPrivate->pAppSettings, APP_BP_SHOW_TOOLBAR))
+      gtk_widget_show(pPrivate->pToolbar);
+    else
+      gtk_widget_hide(pPrivate->pToolbar);
     gtk_widget_show(pPrivate->pMenuBar);
-    //    gtk_window_set_decorated(GTK_WINDOW(pPrivate->pMainWindow), true);
-    //    gtk_frame_set_shadow_type(GTK_FRAME(pPrivate->pOuterFrame), GTK_SHADOW_NONE);
-
-//     gtk_widget_ref(pPrivate->pOuterFrame);
-//     gtk_container_remove(GTK_CONTAINER(pPrivate->pMainWindow), pPrivate->pOuterFrame);
-
-//     gtk_widget_reparent(pPrivate->pInnerFrame, pPrivate->pMainWindow);
   }
+  
+  if(dasher_app_settings_get_bool(pPrivate->pAppSettings, BP_SHOW_SLIDER))
+    gtk_widget_show(pPrivate->pStatusControl);
+  else
+    gtk_widget_hide(pPrivate->pStatusControl);
 
   if(pPrivate->bFullScreen) {
     gtk_window_fullscreen(GTK_WINDOW(pPrivate->pMainWindow));
@@ -674,13 +795,12 @@ void dasher_main_setup_window_style(DasherMain *pSelf, bool bTopMost) {
 // }
 
 void dasher_main_on_map(DasherMain *pSelf) {
-
-  g_message("In map routine");
-
   DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
 
   // Refresh the properties of the window
 
+  bool bDock(dasher_app_settings_get_bool(g_pDasherAppSettings, APP_BP_DOCK));
+
   switch(dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_STYLE)) {
   case 0:
     pPrivate->bShowEdit = true;
@@ -694,14 +814,14 @@ void dasher_main_on_map(DasherMain *pSelf) {
     pPrivate->bShowActions = true;
     pPrivate->bTopMost = true;
     pPrivate->bFullScreen = false;
-    pPrivate->bShowSideMenu = true;
+    pPrivate->bShowSideMenu = bDock;
     break;
   case 2:
     pPrivate->bShowEdit = false;
     pPrivate->bShowActions = false;
     pPrivate->bTopMost = true;
     pPrivate->bFullScreen = false;
-    pPrivate->bShowSideMenu = true;
+    pPrivate->bShowSideMenu = bDock;
     break;
   case 3:
     pPrivate->bShowEdit = true;
@@ -714,7 +834,10 @@ void dasher_main_on_map(DasherMain *pSelf) {
   }
 
 #ifdef WITH_MAEMO
+
+  // TODO: Move more of this stuff into the Maemo helper class
   
+#ifndef WITH_MAEMOFULLSCREEN
   Window xThisWindow = GDK_WINDOW_XWINDOW(pPrivate->pMainWindow->window);
   Atom atom_im_window = gdk_x11_get_xatom_by_name("_HILDON_IM_WINDOW");
   
@@ -748,6 +871,8 @@ void dasher_main_on_map(DasherMain *pSelf) {
   }
 
   dasher_maemo_helper_setup_window(pPrivate->pMaemoHelper);
+#endif
+
 #else
   dasher_main_setup_window_style(pSelf, false);
   dasher_main_setup_window_position(pSelf);
@@ -756,13 +881,8 @@ void dasher_main_on_map(DasherMain *pSelf) {
 
 static void dasher_main_setup_window_type(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
-
-  std::cout << "App style: " << dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_STYLE) << std::endl;
   
   if(((dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_STYLE) == 1) || (dasher_app_settings_get_long(g_pDasherAppSettings, APP_LP_STYLE) == 2)) && dasher_app_settings_get_bool(g_pDasherAppSettings, APP_BP_DOCK)) {
-
-    std::cout << "Mapped: " << GTK_WIDGET_MAPPED(pPrivate->pMainWindow) << std::endl;
-
     gtk_window_set_type_hint(GTK_WINDOW(pPrivate->pMainWindow), GDK_WINDOW_TYPE_HINT_DOCK);
   }
   else {
@@ -787,22 +907,47 @@ void dasher_main_toggle_hidden(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
   pPrivate->bHidden = !(pPrivate->bHidden);
 
+  if(pPrivate->bHidden)
+    gtk_arrow_set(GTK_ARROW(pPrivate->pHideArrow), GTK_ARROW_LEFT, GTK_SHADOW_NONE);
+  else
+    gtk_arrow_set(GTK_ARROW(pPrivate->pHideArrow), GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+
   dasher_main_on_map(pSelf);
 }
 
 void dasher_main_grab(DasherMain *pSelf, GdkEventButton *pEvent) {
   DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
 
-  pPrivate->bGrabbed = true;
-
-  int iWindowX;
-  int iWindowY;
-
-  gtk_window_get_position(GTK_WINDOW(pPrivate->pMainWindow), &iWindowX, &iWindowY);
-
-  pPrivate->dDragOffsetX = pEvent->x_root - iWindowX;
-  pPrivate->dDragOffsetY = pEvent->y_root - iWindowY;
+  if(pEvent->button == 3) {
+    // Right click -> context menu
 
+    gtk_menu_popup(GTK_MENU(pPrivate->pContextMenu), NULL,
+		   NULL, NULL, NULL,
+		   pEvent->button, pEvent->time);
+  }
+  else {
+    // Left / middle click -> drag dock
+    pPrivate->bGrabbed = true;
+    
+    int iWindowX;
+    int iWindowY;
+    
+    gtk_window_get_position(GTK_WINDOW(pPrivate->pMainWindow), &iWindowX, &iWindowY);
+    
+    pPrivate->dDragOffsetX = pEvent->x_root - iWindowX;
+    pPrivate->dDragOffsetY = pEvent->y_root - iWindowY;
+    
+    GdkCursor *pCursor = gdk_cursor_new(GDK_FLEUR);
+    
+    gdk_pointer_grab(pPrivate->pMainWindow->window,
+		     TRUE,
+		     (GdkEventMask)(GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK),
+		     NULL,
+		     pCursor,
+		     pEvent->time);
+    
+    gdk_cursor_unref(pCursor);
+  }
 }
 
 void dasher_main_ungrab(DasherMain *pSelf, GdkEventButton *pEvent) {
@@ -810,6 +955,8 @@ void dasher_main_ungrab(DasherMain *pSelf, GdkEventButton *pEvent) {
 
   pPrivate->bGrabbed = false;
   dasher_main_save_state(pSelf);
+
+  gdk_pointer_ungrab(pEvent->time);
 }
 
 gboolean dasher_main_motion(DasherMain *pSelf, GdkEventMotion *pEvent) {
@@ -910,6 +1057,34 @@ gboolean dasher_main_alphabet_combo_changed(DasherMain *pSelf) {
     dasher_app_settings_set_string(pPrivate->pAppSettings, SP_ALPHABET_ID, szSelected);
 }
 
+void dasher_main_build_context_menu(DasherMain *pSelf) {
+  DasherMainPrivate *pPrivate = (DasherMainPrivate *)(pSelf->private_data);
+
+  pPrivate->pContextMenu = gtk_menu_new();
+
+  GtkWidget *pNewItem;
+
+  pNewItem = gtk_menu_item_new_with_label("Prefences...");
+  gtk_widget_show(pNewItem);
+  gtk_menu_shell_append(GTK_MENU_SHELL(pPrivate->pContextMenu), pNewItem);
+  g_signal_connect(G_OBJECT(pNewItem), "activate", G_CALLBACK(preferences_display), NULL);
+
+  pNewItem = gtk_menu_item_new_with_label("About...");
+  gtk_widget_show(pNewItem);
+  gtk_menu_shell_append(GTK_MENU_SHELL(pPrivate->pContextMenu), pNewItem);
+  g_signal_connect(G_OBJECT(pNewItem), "activate", G_CALLBACK(about_dasher), NULL);
+
+
+  pNewItem = gtk_separator_menu_item_new();
+  gtk_widget_show(pNewItem);
+  gtk_menu_shell_append(GTK_MENU_SHELL(pPrivate->pContextMenu), pNewItem);
+
+  pNewItem = gtk_menu_item_new_with_label("Quit");
+  gtk_widget_show(pNewItem);
+  gtk_menu_shell_append(GTK_MENU_SHELL(pPrivate->pContextMenu), pNewItem);  
+  g_signal_connect(G_OBJECT(pNewItem), "activate", G_CALLBACK(ask_save_before_exit), NULL);
+}
+
 // Callbacks
 
 extern "C" GtkWidget *create_dasher_control(gchar *szName, gchar *szString1, gchar *szString2, gint iInt1, gint iInt2) {
